#include "types.h"
#include "select.h"
#include "error.h"
#include "stdlib.h"

#include "stdio.h"

// assist {

//      
// return:
//   1  
//   0   
//   -1  

int readline(char* p,int max,FILE* f)
{
  int c;
  while((c=fgetc(f))!=EOF)
  {
    if(c==0xd)
      continue;
    
    if(c==0xa)
    {
      *p=0;
      return 0;
    }
    if(c==';')
    {
      *p=0;
      return 1;
    }
    if(!--max)
    {
      *p=0;
      return 1;
    }
    *p++=c;
  }
  return -1;
}

//    

int readheaders(FILE* f,fieldname* headers,int headersmax)
{
  int i=0;
  int read=1;
  while(read==1)
  {
    read=readline(*headers++,sizeof(fieldname),f);
    if(i>=headersmax)
    {
      seterror("max headers reach");
      return -1;
    }
    i++;
  }
  if(i==0)
  {
    seterror("no headers read");
    return -1;
  }
  return i;
}

//    

int readfield(field* p,FILE* f)
{
  return readline((char*)p,sizeof(field),f);
}

//     .
// selectfieldsidx -  
// selectfieldsidxcount -  
// tbl -  
// *data   (   )
// datamax -  

int readfile(int *selectfieldsidx,int selectfieldsidxcount,tablename *tbl,field *data,int datamax)
{
  int dataleft=datamax;

  FILE *f=fopen(*tbl,"rb");
  fieldname headers[32]={0};
  int headersmax=sizeof(headers)/sizeof(fieldname);
  int i;
  field fld;

  if(f==0)
  {
    seterror("can't open input file");
    return -1;
  }

  if(readheaders(f,headers,headersmax)==-1)
    return -1;

  while(!feof(f))
  {//  
    int read=1;
    i=0;
    while(read==1)
    {
      //    
      read=readfield(&fld,f);
      if(findinnumarray(selectfieldsidx,selectfieldsidxcount,i)!=-1)
      {//    
        if(dataleft--==0)
        {
          seterror("data overflow");
          return -1;
        }
        _strcpy(*data++,fld);
      }
      i++;
    }
  }

  fclose(f);

  return datamax-dataleft;
}

// } assist

int select(int *selectfieldsidx,int selectfieldsidxcount,tablename *tbl,field *data,int datamax)
{
  int total=0;
  while(**tbl)
  {
    int read=readfile(selectfieldsidx,selectfieldsidxcount,tbl,data,datamax);
    if(read==-1)
      return -1;
    data+=read;
    datamax-=read;
    total+=read;
    tbl++;
  }
  return total;
}

int normalizeparams(fieldname* selectfields,fieldname* orderfield,tablename *tbl,int* selectfieldsidx,int *selectfieldscount,int *orderfieldidx)
{
  int ret=1;
  FILE *f=0;
  int i;
  int star=0;
  *orderfieldidx=-1;

  TRACE("normalizeparams enter %08x\n",orderfieldidx);

  {
    fieldname* fld=selectfields;

    i=0;
    while(**fld!=0)
    {
      if(_streq(*fld,"*"))
      {
        star++;
      }
      i++;
      fld++;
    }
    if(star>1)
    {
      seterror("more that one '*'");
      goto error;
    }
    if(star==1&&i!=1)
    {
      seterror("have '*' with other fields");
      goto error;
    }
  }

  TRACE("normalizeparams first while star=%d\n",star);

  while(**tbl!=0)
  {
    fieldname headers[32]={0};
    int headersmax=sizeof(headers)/sizeof(fieldname);
    int headerscount;

    f=fopen(*tbl,"rb");

    if(f==0)
    {
      seterror("can't open input file");
      goto error;
    }

    if((headerscount=readheaders(f,headers,headersmax))==-1)
      goto error;

    if(star==0)
    {
      *selectfieldscount=0;
      for(i=0;i<headerscount;i++)
      {
        int *selectfieldsidx_itr=selectfieldsidx;
        fieldname* fld=selectfields;
        TRACE("normalizeparams '%s'\n",headers[i]);
        while(**fld!=0)
        {
          if(_stringformatcmp(headers[i],*fld)==0)
          {
            *selectfieldsidx_itr++=i;
            (*selectfieldscount)++;
          }
          fld++;
        }
      }
    }else
    {
      int *selectfieldsidx_itr=selectfieldsidx;
      *selectfieldscount=headerscount;

      for(i=0;i<headerscount;i++)
      {
        TRACE("normalizeparams '%s'\n",headers[i]);
        *selectfieldsidx_itr++=i;
      }
    }

    TRACE("normalizeparams before set idx %08x %d\n",&orderfieldidx,*orderfieldidx);

    for(i=0;i<*selectfieldscount;i++)
    {
      TRACE("normalizeparams '%s' == '%s'\n",headers[i],*orderfield);
      TRACE_HEX(headers[i]);
      if(_stringformatcmp(headers[selectfieldsidx[i]],*orderfield)==0)
      {
        TRACE("normalizeparams found '%s'\n",headers[i]);
        *orderfieldidx=selectfieldsidx[i];
      }
    }

    fclose(f);
    f=0;
    tbl++;
  }

  TRACE("normalizeparams before exit %08x %08x %d\n",&orderfieldidx,orderfieldidx,*orderfieldidx);

  if(*orderfieldidx==-1)
  {
    seterror("order field not found");
    goto error;
  }

  TRACE("normalizeparams before goto exit\n");
  goto exit;
error:
  TRACE("normalizeparams error\n");
  ret=0;
exit:
  TRACE("normalizeparams pre exit\n");
  if(f!=0)
    fclose(f);
  TRACE("normalizeparams exit\n");

  return ret;
}
